import os
import numpy as np

from typing import List, Optional

try:
    from SkateTracker.art import ascii_art
except ModuleNotFoundError:
    from art import ascii_art


from rich.table import Table
from rich.panel import Panel
from rich.markdown import Markdown


def ordinal(n: int):
    """Utility function that gives the correct string ordinal given a number. For example, 1 gives 1st, 2 give 2nd,
     14 gives 14th etc... From: https://stackoverflow.com/questions/9647202/ordinal-numbers-replacement

    :param int n: integer that the user wants to convert to an ordinal string
    :return:
    """
    return "%d%s" % (n, "tsnrhtdd"[(n // 10 % 10 != 1) * (n % 10 < 4) * n % 10::4])


def update_best_times(names: List[str], times: np.array, best_names: List[str], best_times: np.array) -> np.array:
    """Utility function that takes the new times and best times as input and updates the top 3 best times according
     to the new times.

    :param List[str] names: List of strings with the names of the athletes.
    :param np.array times: Numpy array with the times so far.
    :param List[str] best_names: List of strings with the names of the best athletes so far.
    :param np.array best_times: Numpy array with the best times so far.
    :return np.array: Returns a Numpy array with the new top 3 times, shape=(3, nr_laps)
    """
    all_names = np.array(names + best_names, dtype=object)
    all_times = np.vstack((times, best_times))

    total_times = all_times.sum(axis=1)
    total_times[total_times == 0] = np.infty
    total_times_sorted = total_times.argsort()
    return list(all_names[total_times_sorted[:3]]), all_times[total_times_sorted[:3]]


def check_saved(tournament: str, gender: str, length: int) -> str:
    """Check if there already exists file for this particular race

    :param str tournament: string with the name of the tournament where the race is being held.
    :param str gender: string indicating if the race is for Men or Women. Accepted values are ["M", "F"]
    :param int length: integer indicating the length of the race. Accepted values are
     [500, 1000, 1500, 3000, 5000, 10000]

    :return str: Return a string ("y", "n", "no file") Indicating if a file was found if it is going to be used
    """
    gender_name = "Men" if gender == "M" else "Women"

    if os.path.isfile(os.path.join(os.getcwd(), 'skate_data', f"{tournament}_race_{gender_name}_{length}m_data.csv")):
        correct = False
        while not correct:
            use = input("Data for this race was already found. Do you want to use it? [y/n]")
            if not (use == "y" or use == "n"):
                print("ERROR: please input a valid response!")
            else:
                correct = True
    else:
        use = "no file"
    return use


def save_results(tournament: str, gender: str, length: int, names: List[str], times: np.array):
    """Saves the current names and times.

    :param str tournament: string with the name of the tournament where the race is being held.
    :param str gender: string indicating if the race is for Men or Women. Accepted values are ["M", "F"]
    :param int length: integer indicating the length of the race. Accepted values are
     [500, 1000, 1500, 3000, 5000, 10000]

    :param List[str] names: List of strings with the names of the athletes.
    :param np.array times: Numpy array with the times so far.
    :return None:
    """
    if not os.path.isdir(os.path.join(os.getcwd(), 'skate_data')):
        os.mkdir(os.path.join(os.getcwd(), 'skate_data'))

    gender_name = "Men" if gender == "M" else "Women"

    np.savetxt(
        os.path.join(os.getcwd(), 'skate_data', f"{tournament}_race_{gender_name}_{length}m_data.csv"),
        times,
        header=",".join(names),
        delimiter=",",
        comments="",
        fmt="%1.2f"
    )


def load_results(
        tournament: str,
        gender: Optional[str] = None,
        length: Optional[int] = None,
        fname: Optional[str] = None) -> (List[str], np.array):
    """Loads the results if a specific file is specified, or from a given set of parameters

    :param str tournament:
    :param (str, None) fname: string of a previously recorded race.
    :param (str, None) gender: string indicating if the race is for Men or Women. Accepted values are ["M", "F"]
    :param (int, None) length: integer indicating the length of the race. Accepted values are
     [500, 1000, 1500, 3000, 5000, 10000]

    :return (List[str], np.array): Returns the names of all the athletes aas a list and all the results as a Numpy array
    """
    if fname is not None:
        with open(fname) as f:
            names = f.readline().rstrip().split(",")
        results = np.loadtxt(fname, delimiter=",", skiprows=1)
    else:
        if gender is not None and length is not None:
            gender_name = "Men" if gender == "M" else "Women"
            fname = os.path.join(os.getcwd(), 'skate_data', f"race_{gender_name}_{length}m_data.csv")
            with open(fname) as f:
                names = f.readline().rstrip().split(",")
            results = np.loadtxt(fname, delimiter=",", skiprows=1)
        else:
            raise ValueError("ERROR: Please insert a valid gender or length")
    return names, results.T


class Header:
    """Display header with clock."""

    def __rich__(self) -> Panel:
        grid = Table.grid(expand=True)
        grid.add_column(justify="right", width=5)
        grid.add_column(justify='right', width=40, vertical="middle")
        grid.add_column(justify="center", ratio=1)
        grid.add_row(
            " ",
            Markdown("# Welcome to this CLI speed skate race tracker!\n"),
            ascii_art
        )
        return Panel(grid, style="bold white on blue")


"""
CLI tool (python module) that helps you track lap times during a speed skating race.
It will also predict the final time, as well as visualise some useful  statistics
"""
import argparse
from math import ceil

from rich import print

try:
    from SkateTracker.layout import *
    from SkateTracker.utils import *
    from SkateTracker.plot import create_plotext_panel
except ModuleNotFoundError:
    from layout import *
    from utils import *
    from plot import create_plotext_panel


def welcome_print(tournament: str, gender: str, length: int, prediction_method: str, accumulate: str, save: str) -> Panel:
    """Prints a welcome message showing what parameters have been set and wat will happen next.

    :param str tournament: string with the name of the tournament where the race is being held.
    :param str gender: string indicating if the race is for Men or Women. Accepted values are ["M", "F"]
    :param int length: integer indicating the length of the race. Accepted values are
     [500, 1000, 1500, 3000, 5000, 10000]

    :param str prediction_method: string indicating which prediction method to use. NOT IMPLEMENTED YET
    :param str accumulate: string which indicates if all previous results should also be shown. Accepted values
     ["y", "n"]. NOT IMPLEMENTED YET

    :param str save: string which indicates if the results after each race should be saved to a csv file.
     If the user exits the tool and restarts it again with the same settings, it will use this file to load the previous
     results type

    :return rich.Panel: returns a rich.Panel with the welcome message
    """
    gender_name = "Men" if gender == "M" else "Women"
    accumulated = "be accumulated" if accumulate == "y" else "not be accumulated"
    save_text = "be saved" if save == "y" else "not be saved"

    panel = Panel(
        f"I will start tracking the race with the following parameters:\n\n"
        f"\t-- {gender_name}'s {length}m race in {tournament},\n"
        f"\t-- Predictions will be made with {prediction_method},\n"
        f"\t-- The results will {accumulated},\n"
        f"\t-- The results will {save_text}.\n\n",
        title="CLI speed skate race tracker",
        padding=(2, 2),
        border_style="green"
    )

    return panel


def make_instruction_panel() -> Panel:
    """Prints the instruction message showing what the user has to do next

    :return rich.Panel: returns a rich.Panel with the instruction message
    """
    panel = Panel(
        Align.center(
            f"During every race, you will be able to track one athlete or both athletes.\n\n"
            f"Indicate which one you want by typing the name(s) of the athlete(s) in a comma separated fashion",
            vertical="middle"
        ),
        title="Instructions",
        padding=(2, 2),
        border_style="red",
    )
    return panel


def create_race_view(
        gender: str,
        names: List[str],
        times: np.array,
        nr_laps: int,
        best_names: List[str],
        best_times: np.array,
        race_layout: Layout,
        first: bool = False,
        final: bool = False) -> None:
    """Creates the main race view and layout.

    :param str gender: string indicating if the race is for Men or Women. Accepted values are ["M", "F"].
    :param List[str] names: List of strings with the names of the athletes.
    :param np.array times: Numpy array with the times so far.
    :param int nr_laps: integer indicating how many laps this race is going to take.
    :param List[str] best_names: List of strings with the names of the best athletes so far.
    :param np.array best_times: Numpy array with the best times so far.
    :param rich.Layout race_layout: rich.Layout object with the prescribed layout for all the tables and plots
    :param bool first: Boolean indicating if this is the first time making the view. Otherwise, the progress bar will
     be advanced instead of initiated.

    :param bool final: Boolean indicating if this is the final time the view will be shown. THe progress bar does not
     need to be progressed or initiated then.

    :return None: Nothing
    """

    # Extract all the sub-layouts from the main race layout
    plotext_layout = race_layout["plotext"]
    current_race_layout = race_layout["main"]["current_race"]
    best_results_layout = race_layout["main"]["best_results"]
    progress_layout = race_layout["progress"]

    if first:
        # For the first iteration, the progress panel needs to be created and advanced
        progress_panel = create_progress_panel(nr_laps)
        progress_layout.update(progress_panel)
    else:
        if not final:
            # If this iteration is not the firs or the last, then we advance the progress bar
            progress_panel = progress_layout.renderable
            progress_panel.renderable.renderable.advance(progress_panel.renderable.renderable.task_ids[0])
            progress_layout.update(progress_panel)

    _ = create_plotext_panel(
        gender,
        nr_laps,
        names,
        times,
        plotext_layout
    )

    table_progression = create_lap_time_table(names, times, best_times)
    table_best_results = create_best_table(best_names, best_times)

    current_race_layout.update(table_progression)
    best_results_layout.update(table_best_results)


def track_race(
        gender: str,
        names: List[str],
        nr_laps: int,
        best_names: List[str],
        best_times: np.array) -> (np.array, Layout):
    """For each race, this function does the tracking of the race. It will create the race layout and view. For each
     lap it will ask the user for the lap times and incorporate them into the views

    :param str gender: string indicating if the race is for Men or Women. Accepted values are ["M", "F"]
    :param List[str] names: List of strings with the names of the athletes.
    :param int nr_laps: integer indicating how many laps this race is going to take.
    :param List[str] best_names: List of strings with the names of the best athletes so far.
    :param np.array best_times: Numpy array with the best times so far.
    :return (np.array, rich.Layout): Returns the final lap times of the athletes.
     So that they may used to save the times and update the best times. Also returns the layout so that all the results
     can be shown one last time.
    """
    n_athletes = len(names)
    times = np.zeros((len(names), nr_laps))

    race_layout = make_race_layout()
    create_race_view(
        gender,
        names,
        times,
        nr_laps,
        best_names,
        best_times,
        race_layout,
        first=True
    )
    print(race_layout)

    for i in range(nr_laps):
        correct = False
        while not correct:
            try:
                lap_time = np.array([float(t) for t in input(f"Lap times of {ordinal(i+1)} lap:").replace(" ", "").split(",")])
            except ValueError:
                print("ERROR: You need to enter numbers!")
            else:
                if lap_time.shape[0] != n_athletes:
                    print("ERROR: you need to enter the correct amount of times!")

                else:
                    correct = True
                    times[:, i] = lap_time

                # For now the view is erased and updated each time. The reason being that rich.Live does not work
                # nicely with input and a progress bar. (As far as I could figure out)
                os.system("clear")
                create_race_view(
                    gender,
                    names,
                    times,
                    nr_laps,
                    best_names,
                    best_times,
                    race_layout
                )
                print(race_layout)
    return times, race_layout


def main_tracking(tournament: str, gender: str, length: int, prediction_method: str, accumulate: str, save: str):
    """Main function that parses the initial arguments. Prompts the user for the names of the athletes and calls
     all the other functions that create the layouts, views, tracking etc.

    :param str tournament: string with the name of the tournament where the race is being held.
    :param str gender: string indicating if the race is for Men or Women. Accepted values are ["M", "F"]
    :param int length: integer indicating the length of the race. Accepted values are
     [500, 1000, 1500, 3000, 5000, 10000]

    :param str prediction_method: string indicating which prediction method to use. NOT IMPLEMENTED YET
    :param str accumulate: string which indicates if all previous results should also be shown. Accepted values
     ["y", "n"]. NOT IMPLEMENTED YET

    :param str save: string which indicates if the results after each race should be saved to a csv file.
     If the user exits the tool and restarts it again with the same settings, it will use this file to load the previous
     results type

    :return None:
    """

    # verify if all the arguments are correct (some combinations are not possible)
    if gender == "M" and length == 3000:
        raise ValueError("This combination is not a valid race!")
    if gender == "F" and length == 10000:
        raise ValueError("This combination is not a valid race!")

    use = check_saved(tournament, gender, length)
    # Creates the main start layout
    layout_start = make_start_layout()

    # Below the sub-layouts are extracted and updated for the welcome view
    layout_start["header"].update(Header())

    welcome_panel = welcome_print(tournament, gender, length, prediction_method, accumulate, save)
    instruction_panel = make_instruction_panel()
    layout_start["main"]["side"].update(instruction_panel)
    layout_start["main"]["body"].update(welcome_panel)
    print(layout_start)

    nr_laps = ceil(length / 400)
    best_names = ["None", "None", "None"]
    best_times = np.zeros((3, nr_laps))

    if use == "y":
        all_names, all_results = load_results(tournament, gender=gender, length=length)
        best_names, best_times = update_best_times(all_names, all_results, best_names, best_times)
    else:
        all_names = []
        all_results = []

    tracking = True
    while tracking:
        names = input("The names of the athletes are: ").replace(" ", "").split(",")
        # A full race is tracked, the lap times and final race view are returned
        lap_times, race_layout = track_race(gender, names, nr_laps, best_names, best_times)
        best_names, best_times = update_best_times(names, lap_times, best_names, best_times)

        if save == "y":
            if len(all_names) != 0:
                all_results = np.vstack((all_results, lap_times))
                all_names = all_names + names
                save_results(tournament, gender, length, all_names, all_results.T)
            else:
                all_results = lap_times
                all_names = names
                save_results(tournament, gender, length, all_names, all_results.T)

        correct = False
        while not correct:
            next_race = input("Do you want to start tracking the next race? [y/n]: ")
            if not (next_race == "y" or next_race == "n"):
                print("ERROR: please select a valid response!")
            else:
                correct = True

        if next_race == "n":
            create_race_view(
                gender,
                names,
                lap_times,
                nr_laps,
                best_names,
                best_times,
                race_layout,
                final=True
            )
            print(race_layout)
            input("Input anything to quit the Tracker!")
            tracking = False

    return 0


def main():
    parser = argparse.ArgumentParser(
        description="CLI tool to track and predict Speed Skating races"
    )

    parser.add_argument('--tournament', '-t', type=str, required=True,
                        help="The name of the tournament where the race is taking place. "
                             "Will be used in the saved file.")
    parser.add_argument('--gender', '-g', type=str, choices=["M", "F"], required=True,
                        help="Indicate if the race is for the Men (M) or the Women (W).")
    parser.add_argument('--length', '-l', type=int, choices=[500, 1000, 1500, 3000, 5000, 10000], required=True,
                        help="Indicate the length of the race.")
    parser.add_argument("--save", "-s", choices=["y", "n"], type=str, default="y",
                        help="Indicate if all the results should be saved.")
    # TODO: implement these arguments
    parser.add_argument('--prediction_method', '-pm', nargs="+", type=str,
                        choices=["mean", "latest", "lr", "LSTM", "online"],
                        help="Indicate which method to use to predict the ending time.")
    parser.add_argument("--accumulate", "-a", choices=["y", "n"], type=str, default="y",
                        help="Indicate if previous results should be accumulated in the visualisations.")
    # For now this will be a simple csv, might change into something like an sqlite db

    args = parser.parse_args()
    kwargs = vars(args)

    main_tracking(**kwargs)

# Press the green button in the gutter to run the script.
if __name__ == '__main__':
    main()



from datetime import timedelta

from typing import List

import numpy as np

from rich.align import Align
from rich.panel import Panel
from rich.layout import Layout
from rich.table import Table
from rich.progress import Progress, SpinnerColumn, BarColumn, TextColumn


def make_start_layout() -> Layout:
    """Define the layout for the starting screen."""
    layout = Layout(name="root")

    layout.split(
        Layout(name="header", ratio=4),
        Layout(name="main", ratio=3),
    )
    layout["main"].split_row(
        Layout(name="side"),
        Layout(name="body", ratio=2, minimum_size=60),
    )
    return layout


def make_race_layout() -> Layout:
    """Defines the layout of the race tracking screen."""
    layout = Layout(name="root")
    layout.split(
        Layout(name="header", size=3),
        Layout(name="main", ratio=1),
        Layout(name="progress", size=3)
    )
    layout["main"].split_row(
        Layout(name="plotext", ratio=7),
        Layout(name="current_race", ratio=4),
        Layout(name="best_results", ratio=3)
    )
    layout["header"].update(Panel("", border_style="navy_blue"))

    return layout


def create_lap_time_table(names: List[str], times: np.array, top_3_times: np.array) -> Panel:
    """Creates a table which shows the progress of the current races. It has 4 columns. 2 groups of two, with the lap
    times of an athlete and the difference with the current best time.

    :param List[str] names: List of strings with the names of the athletes.
    :param np.array times: Numpy array with the times so far.
    :param np.array top_3_times: Numpy array with the best times so far
    :return rich.Panel: Returns a rich.Panel with the table in it
    """
    table_lap_times = Table(
        title="Lap time progression",
        show_footer=True,
    )

    total_times = times.sum(axis=1)
    table_lap_times.add_column(
        names[0],
        justify="center",
        no_wrap=True,
        min_width=10,
        header_style="red",
        style="red",
        footer=f"{str(timedelta(seconds=total_times[0]))[2:10]}"
    )
    table_lap_times.add_column("Best diff", justify="center", no_wrap=True, min_width=10)
    table_lap_times.add_column(
        names[1],
        justify="center",
        no_wrap=True,
        min_width=10,
        header_style="blue",
        style="blue",
        footer=f"{str(timedelta(seconds=total_times[1]))[2:10]}"
    )
    table_lap_times.add_column("Best diff", justify="center", no_wrap=True, min_width=10)

    best_lap_times = top_3_times[0, :]
    diff = times - best_lap_times
    times = np.vstack((times, diff))

    colors = ("bright_red", "bright_cyan")

    for col in times.T:
        if col[0] == 0 and col[1] == 0:
            table_lap_times.add_row("NA", "NA", "NA", "NA")
        else:
            colors_race = (colors[1 - int(col[0] > col[1])], colors[int(col[0] > col[1])])
            color_diff_1 = colors[1 - int(col[2] > 0)]
            color_diff_2 = colors[1 - int(col[3] > 0)]
            item_1 = f"[{colors_race[0]}]{col[0]:.2f}"
            item_2 = f"[{color_diff_1}]{col[2]:.2f}" if col[2] < 0 else f"[{color_diff_1}]+{col[2]:.2f}"
            item_3 = f"[{colors_race[1]}]{col[1]:.2f}"
            item_4 = f"[{color_diff_2}]{col[3]:.2f}" if col[3] < 0 else f"[{color_diff_2}]+{col[3]:.2f}"
            table_lap_times.add_row(item_1, item_2, item_3, item_4)

    panel_lap_times = Panel(
        Align.center(table_lap_times, vertical="top"),
        border_style="bright_red"
    )

    return panel_lap_times


def create_best_table(names: List[str], times: np.array) -> Panel:
    """Creates the table with the top 3 best times so far.

    :param List[str] names: List of strings with the names of the best athletes so far.
    :param np.array times: Numpy array with the best times so far
    :return rich.Panel: Panel with the table showing the top 3 best results so far.
    """
    table_best = Table(
        title="Best times so far",
        show_footer=True)

    total_times = times.sum(axis=1)
    table_best.add_column(
        names[0],
        justify="center",
        no_wrap=True,
        min_width=10,
        style="gold3",
        header_style="gold3",
        footer=f"{str(timedelta(seconds=total_times[0]))[2:10]}"
    )
    table_best.add_column(
        names[1],
        justify="center",
        no_wrap=True,
        min_width=10,
        style="grey74",
        header_style="grey74",
        footer=f"{str(timedelta(seconds=total_times[1]))[2:10]}"
    )
    table_best.add_column(
        names[2],
        justify="center",
        no_wrap=True,
        min_width=10,
        style="orange4",
        header_style="orange4",
        footer=f"{str(timedelta(seconds=total_times[2]))[2:10]}"
    )
    for col in times.T:
        table_best.add_row(*col.astype(str))

    panel_best = Panel(
        Align.center(table_best, vertical="top"),
        border_style="dark_orange"
    )

    return panel_best


def create_progress_panel(nr_laps: int) -> Panel:
    """Creates a panel with the progress bar

    :param int nr_laps: integer indicating how many laps this race is going to take.
    :return rich.Panel: Panel with the progress bar
    """
    race_progress = Progress(
        "{task.description}",
        SpinnerColumn(),
        BarColumn(),
        TextColumn("[progress.percentage]{task.percentage:>3.0f}%")
    )
    race_progress.add_task("[cyan]Progress", total=nr_laps)
    progress_panel = Panel(
        Align.center(race_progress, vertical="middle"),
        border_style="cyan"
    )
    return progress_panel




import numpy as np
import plotext as plt

from typing import List
from rich.jupyter import JupyterMixin
from rich.ansi import AnsiDecoder
from rich.console import Group as RenderGroup
from rich.layout import Layout
from rich.panel import Panel


def plot_race(gender: str, length: int, names: List[str], lap_times: np.array, *size):
    """Function that takes all parameters and times and produces the two plots in the tracking view. Depends mainly on
     plotext

    :param str gender: string indicating if the race is for Men or Women. Accepted values are ["M", "F"]
    :param int length: integer indicating the length of the race. Accepted values are
     [500, 1000, 1500, 3000, 5000, 10000]

    :param List[str] names: List of strings with the names of the athletes.
    :param np.array lap_times:  Numpy array with the times so far.
    :param size: Additional arguments for the width and the height of the plots
    :return:
    """
    colors = ("red", "blue")

    gender_name = "Men" if gender == "M" else "Women"
    title_names = " vs ".join(names)
    nr_athletes = len(names)
    nr_laps = lap_times.shape[1]
    plt.subplots(2,1)

    total_times = np.cumsum(lap_times, axis=1)
    total_times[lap_times == 0] = 0
    # Plot of the total
    plt.subplot(1, 1)
    for i in range(nr_athletes):
        if total_times[i, :].sum() == 0:
            athlete_times = total_times[i, :]
        else:
            athlete_times = total_times[i, total_times[i, :] > 0]
        plt.plot(athlete_times, color=colors[i], label=names[i])
    plt.plotsize(*size)
    plt.ylim(0, nr_laps * 35)
    plt.title(f"{gender_name}'s {length}m race total times: {title_names}")
    plt.xticks(list(range(nr_laps)))
    plt.xlim(1, nr_laps)

    # Plot of the lap times
    plt.subplot(2, 1)
    for i in range(nr_athletes):
        if total_times[i, :].sum() == 0:
            athlete_times = lap_times[i, :]
        else:
            athlete_times = lap_times[i, lap_times[i, :] > 0]
        plt.plot(athlete_times, color=colors[i])
    plt.plotsize(*size)
    plt.ylim(5, 35)
    plt.title(f"{gender_name}'s {length}m race lap times: {title_names}")
    plt.xticks(list(range(nr_laps)))
    plt.xlim(1, nr_laps)
    return plt.build()


class plotextMixin(JupyterMixin):
    """plotextMixin that allows plotext figures to be placed inside a rich.Layout. Got the code from
     https://github.com/piccolomo/plotext/issues/26
    """
    def __init__(self, gender: str, length: int, names: List[str], lap_times: np.array):
        """Initialize

        :param str gender: string indicating if the race is for Men or Women. Accepted values are ["M", "F"]
        :param int length: integer indicating the length of the race. Accepted values are
         [500, 1000, 1500, 3000, 5000, 10000]

        :param List[str] names: List of strings with the names of the athletes.
        :param np.array lap_times:  Numpy array with the times so far.
        """
        self.decoder = AnsiDecoder()
        self.gender = gender
        self.length = length
        self.names = names
        self.lap_times = lap_times

    def __rich_console__(self, console, options):
        self.width = options.max_width or console.width
        self.height = options.height or console.height
        canvas = plot_race(
            self.gender,
            self.length,
            self.names,
            self.lap_times,
            self.width,
            self.height / 2)
        self.rich_canvas = RenderGroup(*self.decoder.decode(canvas))
        yield self.rich_canvas


def create_plotext_panel(
        gender: str,
        length: int,
        names: List[str],
        lap_times: np.array,
        layout: Layout) -> Panel:
    """Creates the actual ponel with the plotext in it. The layouy that is supplied will be used to put the panel into.

    :param str gender: string indicating if the race is for Men or Women. Accepted values are ["M", "F"]
    :param int length: integer indicating the length of the race. Accepted values are
     [500, 1000, 1500, 3000, 5000, 10000]

    :param List[str] names: List of strings with the names of the athletes.
    :param np.array lap_times:  Numpy array with the times so far.
    :param rich.Layout layout:  The layout in which the plotext figure will be placed.
    :return rich.Panel:  The panel with the plotext in it is returned
    """
    mix = plotextMixin(
        gender,
        length,
        names,
        lap_times
    )
    mix = Panel(mix)
    layout.update(mix)
    return mix


from setuptools import setup

def readme():
    with open('README.md') as f:
        return f.read()


setup(
    name="SkateTracker",
    version="0.1.1",
    description="CLI tool to track speed skating races",
    long_description=readme(),
    long_description_content_type="text/markdown",
    author="Hidde Fokkema",
    url="https://github.com/HiddeFok/CLI_speed_skating_tracker",
    license="MIT",
    keywords="CLI sports visualization",
    packages=[
        "SkateTracker"
    ],
    python_requires=">=3.5",
    install_requires=[
        "numpy==1.22.0",
        "plotext==4.1.5",
        "rich==11.0.0"
    ],
    entry_points={
        "console_scripts": [
            "SkateTracker = SkateTracker.main:main"
        ]
    }
)